package org.jvending.provisioning.impl;

import org.jvending.provisioning.config.stockinghandlers.*;

import java.util.List;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.Collections;
import java.util.logging.Logger;

final class StockingPolicyFactory {

    private static Logger logger = Logger.getLogger("StockingPolicyFactory");

    private StockingPolicyFactory() { }

    public static org.jvending.provisioning.stocking.StockingPolicy createStockingPolicy(PolicyType policyType) {
        return PolicyTranslator.translate(policyType);
    }

    private static class PolicyTranslator {

        private static int defaultRemoteMaxSize = 100;

        private static int defaultLocalMaxSize = 500;

        static org.jvending.provisioning.stocking.StockingPolicy translate(PolicyType policyType) {
            if(policyType == null) return null;

            int remoteMaxSize = (policyType.getRemoteMaxSize() != null) ?
                policyType.getRemoteMaxSize().intValue(): defaultRemoteMaxSize;

            int localMaxSize = (policyType.getLocalMaxSize() != null) ?
                policyType.getLocalMaxSize().intValue(): defaultLocalMaxSize;

            boolean hasRemoteFetch = policyType.isFetchContent();

            MimeAcceptType mimeAcceptType = policyType.getMimeAccept();
            List mimeAccepts = (mimeAcceptType != null) ? translateMimeAccept(mimeAcceptType): null;

            MimeBlockType mimeBlockType = policyType.getMimeBlock();
            List mimeBlocks = (mimeBlockType != null) ? translateMimeBlock(mimeBlockType): null;

            WhiteListType whiteListType = policyType.getWhiteList();
            List whiteList = (whiteListType != null) ? translateWhiteList(whiteListType): null;

            BlackListType blackListType = policyType.getBlackList();
            List blackList = (blackListType != null) ? translateBlackList(blackListType): null;

            org.jvending.provisioning.stocking.StockingPolicy stockingPolicy =
                new StockingPolicyImpl(remoteMaxSize, localMaxSize, hasRemoteFetch,
                                       mimeAccepts, mimeBlocks, whiteList, blackList);
            return stockingPolicy;
        }

        static List translateWhiteList(WhiteListType uriList) {
            List uris = (uriList.getUri() != null) ? uriList.getUri(): new ArrayList();
            if(uris.size() == 0) {
                uris.add("[.]");
                logger.finest("JV-1851-001: Adding all values to white list");
                return uris;
            }
            return getStringList(uris);
        }

        static List translateBlackList(BlackListType uriList) {
            List uris = (uriList.getUri() != null) ? uriList.getUri(): new ArrayList();
            return getStringList(uris);
        }

        static List translateMimeAccept(MimeAcceptType mime) {
            List mimeTypes = (mime != null) ? mime.getMimeType(): new ArrayList();
            if(mimeTypes.size() == 0) {
                mimeTypes.add("[.]");
                logger.finest("JV-1851-002: Adding all values to mime types");
                return mimeTypes;
            }
            return getStringList(mimeTypes);
        }

        static List translateMimeBlock(MimeBlockType mime) {
            List mimeTypes = (mime != null) ? mime.getMimeType(): new ArrayList();
            return getStringList(mimeTypes);
        }

        static List getStringList(List base) {
            List returns = new ArrayList();

            for(Iterator i = base.iterator(); i.hasNext(); ) {
                String s = (String) i.next();
                if(s != null) returns.add(s);
            }
            return returns;
        }

        private static class StockingPolicyImpl implements org.jvending.provisioning.stocking.StockingPolicy {

            private final int remoteMaxSize;

            private final int localMaxSize;

            private final boolean hasFetchContent;

            private final List mimeAccept;

            private final List mimeBlock;

            private final List whiteList;

            private final List blackList;

            StockingPolicyImpl(int remoteMaxSize, int localMaxSize, boolean hasFetchContent,
                               List mimeAccept, List mimeBlock, List whiteList, List blackList) {
                this.remoteMaxSize = remoteMaxSize;
                this.localMaxSize = localMaxSize;
                this.hasFetchContent = hasFetchContent;
                this.mimeAccept = (mimeAccept != null) ? Collections.unmodifiableList(mimeAccept):
                    Collections.unmodifiableList(new ArrayList());
                this.mimeBlock = (mimeBlock != null) ? Collections.unmodifiableList(mimeBlock):
                    Collections.unmodifiableList(new ArrayList());
                this.whiteList = (whiteList != null) ? Collections.unmodifiableList(whiteList):
                    Collections.unmodifiableList(new ArrayList());
                this.blackList = (blackList != null) ? Collections.unmodifiableList(blackList):
                    Collections.unmodifiableList(new ArrayList());
            }

            public int getRemoteMaxSize() {
                return remoteMaxSize;
            }

            public int getLocalMaxSize() {
                return localMaxSize;
            }

            public boolean hasFetchContent() {
                return hasFetchContent;
            }

            public List getMimeAccept() {
                return mimeAccept;
            }

            public List getMimeBlock() {
                return mimeBlock;
            }

            public List getWhiteList() {
                return whiteList;
            }

            public List getBlackList() {
                return blackList;
            }
        }
    }
}
