package org.jvending.provisioning.impl;

import junit.framework.TestCase;
import org.jvending.provisioning.model.clientbundle.ClientBundle;
import org.jvending.provisioning.stocking.par.ClientBundleType;
import org.jvending.provisioning.stocking.par.ProvisioningArchiveType;
import org.jvending.provisioning.stocking.filter.FormatFilter;
import org.jvending.provisioning.stocking.StockingContext;
import org.jvending.provisioning.stocking.StockingException;
import org.jvending.provisioning.stocking.handler.StockingHandlerConfig;
import org.jvending.provisioning.config.MimeTypeRepository;
import org.jvending.provisioning.config.DeliverableRepository;
import org.jvending.provisioning.config.StockingHandlersRepository;

import org.jvending.registry.RepositoryRegistry;
import org.jvending.registry.jaxb.JaxbConfiguration;
import org.easymock.MockControl;

import javax.provisioning.BundleDescriptor;
import javax.provisioning.BundleType;
import javax.provisioning.DescriptorFile;
import javax.provisioning.ProvisioningException;
import javax.servlet.ServletContext;

import java.util.Map;
import java.util.HashMap;
import java.util.List;
import java.util.Locale;
import java.io.FileInputStream;
import java.io.IOException;

public class BundleDescriptorImplTest extends TestCase {

    private static String baseDir = System.getProperty("basedir");

    private BundleDescriptor bundleDescriptor;

    public void setUp() {
        RepositoryRegistry repositoryRegistry = RepositoryRegistry.Factory.create();
        try {
            repositoryRegistry.loadFromFile(baseDir + "/src/test/resources/registry-config.xml");
        } catch (IOException e) {
            e.printStackTrace();
           // fail("Could not load the registry-config file");
        }

        ClientBundleType bundleType = null;
        try {
            FileInputStream fis = new FileInputStream(baseDir + "/src/test/resources/provisioning.xml");
            ProvisioningArchiveType archiveType =
                    (ProvisioningArchiveType) JaxbConfiguration.parse(fis ,"org.jvending.provisioning.stocking.par");
            List clientBundles = archiveType.getClientBundle();
            bundleType = (ClientBundleType) clientBundles.get(0);
            FormatFilter filter = new FormatFilter();
            filter.formatClientBundle(bundleType);//remove white space
        } catch (IOException e) {
          //  fail("Could not parse provisioning.xml file");
        }

        Map contentMap = new HashMap();
        String jadString = "a: b";
        contentMap.put("games.jad", jadString.getBytes());
        MimeTypeRepository mimeTypeRepository = (MimeTypeRepository) repositoryRegistry.find("mimetype");
        DeliverableRepository deliverableRepository = (DeliverableRepository) repositoryRegistry.find("deliverables");
        StockingHandlersRepository stockingHandlersRepository =
                (StockingHandlersRepository) repositoryRegistry.find("stocking-handlers");

        if(stockingHandlersRepository == null) fail("Could not find StockingHandlersRepository");

        MockControl controlServletContext = MockControl.createNiceControl(ServletContext.class);
        ServletContext servletContext = (ServletContext) controlServletContext.getMock();

        servletContext.getAttribute("org.jvending.registry.RepositoryRegistry");
        controlServletContext.setReturnValue(repositoryRegistry);

        servletContext.getAttribute("org.jvending.registry.RepositoryRegistry");
        controlServletContext.setReturnValue(repositoryRegistry);        

        controlServletContext.replay();

        StockingContext stockingContext = new StockingContextImpl();
        stockingContext.init(servletContext);

        StockingHandlerConfig config = null;
        try {
            config = stockingContext.getStockingHandler("GENERIC").getStockingHandlerConfig();
        } catch (StockingException e) {
            e.printStackTrace();
            fail("Could not obtain the StockingHandlerConfig. Name = GENERIC");
        }

        ClientBundle clientBundle = null;
        try {
            clientBundle = ClientBundleTranslator.translate(bundleType, contentMap, config,
                mimeTypeRepository);
        } catch(ProvisioningException e) {
            fail("Failed to translate ClientBundle: Message = " + e.getMessage());
        }
        bundleDescriptor = new BundleDescriptorImpl(clientBundle);
        ((BundleDescriptorImpl) bundleDescriptor).setDeliverableRepository(deliverableRepository);
        ((BundleDescriptorImpl) bundleDescriptor).setMimeTypeRepository(mimeTypeRepository);
        ((BundleDescriptorImpl) bundleDescriptor).setContentDeliveryUri("http://localhost:8080/jvending");
    }

    public void testContentId() {
        assertEquals("http://java.sun.com/products/j2mewtoolkit/apps/games", bundleDescriptor.getContentID());
    }

    public void testBundleType() {
        assertEquals(BundleType.APPLICATION, bundleDescriptor.getBundleType());
    }

    public void testVersion() {
        assertEquals("2.4", bundleDescriptor.getVersion());
    }

    public void testCopyright() {
        assertEquals("Copyright 2000-2002 Sun Microsystems, Inc. All Rights Reserved.",
                bundleDescriptor.getCopyright(null));
        assertEquals("Copyright (In English) 2000-2002 Sun Microsystems, Inc. All Rights Reserved.",
                bundleDescriptor.getCopyright(new Locale("EN")));
    }

    public void testDescription() {
        assertEquals("Sample suite of games for the MIDP.\n" +
                "        Three games are included: Tile Puzzle, Worm Game and Push Puzzle.",
                bundleDescriptor.getDescription(null));
        assertEquals("Sample suite (In English) of games for the MIDP.\n" +
                "        Three games are included: Tile Puzzle, Worm Game and Push Puzzle.",
                bundleDescriptor.getDescription(new Locale("EN")));
    }

    public void testDescriptorFile() {
        DescriptorFile descriptorFile = (DescriptorFile) bundleDescriptor.getDescriptorFile();
        assertNotNull("Descriptor File is null: " + bundleDescriptor.toString(), descriptorFile);
        assertEquals("Incorrect mime type = " + descriptorFile.getMimeType(),
                descriptorFile.getMimeType(), "text/vnd.sun.j2me.app-descriptor");
        assertNotNull("Descriptor File URL is null", descriptorFile.getURL());
        String url = descriptorFile.getURL().toString();
        assertTrue("Incorrect URL: URL = " + url, url.endsWith("&type=descriptor"));
    }

}
